<xsl:stylesheet version="2.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="xml" indent="yes"/>
<xsl:key name="ndkey" match="nd" use="@ref"/>

<xsl:variable name="osm" select="/osm"/>


<xsl:template match="/osm">
	<xsl:copy>
		<xsl:copy-of select="@*"/>
		<xsl:for-each-group select="way[tag/@k = 'ref']" group-by="tag[@k = 'ref']/@v">
			<xsl:variable name="group">
				<group>
					<xsl:copy-of select="current-group()"/>
				</group>
			</xsl:variable>
			<xsl:apply-templates select="$group" mode="iterateJoin"/>
			<!--
			<gc>
				<xsl:copy-of select="current-grouping-key()"/>
			</gc>
			-->
		</xsl:for-each-group>
	<xsl:copy-of select="node"/>
	</xsl:copy>
</xsl:template>

<xsl:template match="group[*]" mode="iterateJoin">
	<xsl:variable name="joined">
		<xsl:variable name="firstWay">
			<xsl:copy-of select="way[1]/*"/>
		</xsl:variable>
		<xsl:apply-templates select="$firstWay" mode="join">
			<xsl:with-param name="ways" select="way[position() != 1]"/>
			<xsl:with-param name="params" select="way[1]/@*"/>
			<xsl:with-param name="flipAfter" select="'true'"/>
		</xsl:apply-templates>
	</xsl:variable>

	<xsl:copy-of select="$joined/way[1]"/>
	<xsl:variable name="group">
		<group>
			<xsl:copy-of select="$joined/way[position() != 1]"/>
		</group>
	</xsl:variable>
	<xsl:apply-templates select="$group" mode="iterateJoin"/>
</xsl:template>


<xsl:template match="/" mode="join">
	<xsl:param name="ways"/>
	<xsl:param name="params"/>
	<xsl:param name="flipAfter"/>

	<xsl:variable name="lastRef" select="nd[last()]/@ref"/>
	<xsl:variable name="last_all" select="$ways[(nd[1] | nd[last()])/@ref = $lastRef]"/>
	<xsl:variable name="last_w">
		<xsl:for-each select="$last_all[1]">
			<xsl:copy>
				<xsl:copy-of select="@*"/>
				<xsl:if test="nd[1]/@ref != $lastRef">
					<xsl:for-each select="*">
							<xsl:sort select="position()" data-type="number" order="descending"/>
							<xsl:copy-of select="."/>
					</xsl:for-each>
				</xsl:if>
				<xsl:if test="nd[1]/@ref = $lastRef">
					<xsl:copy-of select="*"/>
				</xsl:if>
			</xsl:copy>
		</xsl:for-each>
	</xsl:variable>
	<xsl:variable name="last" select="$last_w/way"/>

	
	<xsl:variable name="dv">
		<xsl:copy-of select="nd"/>
		<xsl:copy-of select="$last/nd[position() != 1]"/>
		<xsl:copy-of select="tag"/>
	</xsl:variable>

	<xsl:apply-templates select="$dv[$last]" mode="join">
		<xsl:with-param name="ways" select="$ways[@id != $last/@id]"/>
		<xsl:with-param name="params" select="$params"/>
		<xsl:with-param name="flipAfter" select="$flipAfter"/>
	</xsl:apply-templates>

	<xsl:if test="not($last) and ($flipAfter = 'true')">
		<xsl:variable name="dv_r">
			<xsl:for-each select="$dv/*">
				<xsl:sort select="position()" data-type="number" order="descending"/>
				<xsl:copy-of select="."/>
			</xsl:for-each>
		</xsl:variable>
		<xsl:apply-templates select="$dv_r" mode="join">
			<xsl:with-param name="ways" select="$ways[not(@id = $last/@id)]"/>
			<xsl:with-param name="params" select="$params"/>
			<xsl:with-param name="flipAfter" select="'false'"/>
		</xsl:apply-templates>
	</xsl:if>

	<xsl:if test="not($last) and ($flipAfter = 'false')">
		<way>
			<xsl:copy-of select="$params | *"/>
		</way>
		<xsl:copy-of select="$ways[@id != $params[name() = 'id']]"/>
	</xsl:if>
</xsl:template>

</xsl:stylesheet>
